@@ -7,15 +7,16 @@ import pymysql as MySQLdb
 import psycopg2
 import math
 import json
-
+import ast
 
 from google.cloud import bigquery
 from google.oauth2 import service_account
 
-
 import abc
 from contextlib import contextmanager
 
+MONGO_OPERATOR_DICT = {"=": "$eq", ">": "$gt", ">=": "$gte", " IN ": "$in", "<": "$lt", "<=": "$lte", "<>": "$ne"}
+
 
 def read_file(file):
     """Reads txt file -> list"""
@@ -23,25 +24,25 @@ def read_file(file):
     with open(file, "r") as f:
         rows = f.readlines()
 
-        for i,row in enumerate(rows):
-            rows[i]=row.replace("\n","")
-    return(rows)
-    
+        for i, row in enumerate(rows):
+            rows[i] = row.replace("\n", "")
+    return (rows)
+
+
 def read_connection_details(config_file):
-    connection_details=read_file(config_file)
-    db_details={}
+    connection_details = read_file(config_file)
+    db_details = {}
     for detail in connection_details:
-        key=detail.split("=")[0]
-        value=detail.split("=")[1]
-        db_details[key]=value
+        key = detail.split("=")[0]
+        value = detail.split("=")[1]
+        db_details[key] = value
 
         # Skip empty lines to avoid error when reading config file
         if not detail:
             continue
 
-    print(", ".join([db_details['DB_SERVER'],db_details['DB_DATABASE'],db_details['DB_USERNAME']]))
-    return(db_details)
-
+    print(", ".join([db_details['DB_SERVER'], db_details['DB_DATABASE'], db_details['DB_USERNAME']]))
+    return (db_details)
 
 
 def save_migration(function, *args, **kw):  # decorator
@@ -79,7 +80,6 @@ def save_migration(function, *args, **kw):  # decorator
     return (new_function)
 
 
-
 class Migrator:
     def __init__(self, db=None):
         self.db = db
@@ -147,7 +147,6 @@ class Migrator:
     def migration_list_to_json(self, filename=None):
         result = json.dumps(self.migration_list)
 
-
         if filename is None or filename == "" or filename.isspace():
             with open("migrations/migration-" + str(self.migration_number) + ".json", "w+") as f:
                 f.write(result)
@@ -157,7 +156,6 @@ class Migrator:
 
     def create_migrations_from_df(self, name, dataframe):
 
-
         columns, return_types = self.extract_columns_and_types_from_df(dataframe)
 
         migration_dict = {"create": {"table_name": name, "columns": columns, "types": return_types}}
@@ -191,11 +189,11 @@ class AbstractDB(abc.ABC):
     def __init__(self, config_file="config.ini", db_details=None):
         if db_details is None:
             db_details = read_connection_details(config_file)
-            
-        self.locally=True
-        if db_details["LOCALLY"]=="False":
-            self.locally=False  
-        
+
+        self.locally = True
+        if db_details["LOCALLY"] == "False":
+            self.locally = False
+
         self.DB_SERVER = db_details["DB_SERVER"]
         self.DB_DATABASE = db_details["DB_DATABASE"]
         self.DB_USERNAME = db_details["DB_USERNAME"]
@@ -213,7 +211,6 @@ class AbstractDB(abc.ABC):
 
         self.connect_to_db()
 
-
     @abc.abstractmethod
     def connect_locally(self):
         pass
@@ -222,14 +219,12 @@ class AbstractDB(abc.ABC):
     def connect_remotely(self):
         pass
 
-
     def _connect(self):
         if self.locally:
             self.connect_locally()
         else:
             self.connect_remotely()
 
-
     def connect(self):
         print("DEPRECATION WARNING: use `connect_to_db` context manager instead of `connect` method")
         if self.locally:
@@ -237,6 +232,27 @@ class AbstractDB(abc.ABC):
         else:
             self.connect_remotely()
 
+    #@abc.abstractmethod
+    def get_all_tables(self):
+        pass
+
+    #@abc.abstractmethod
+    def generate_table_dict(self):
+        pass
+
+    def get_table(self, table_name: str):
+        """
+        Retrieves Table from DB using its name.
+        """
+
+        try:
+            table = self.generate_table_dict()[table_name]
+        except KeyError:
+            print(f'Table "{table_name}" was not found in the DB.')
+            raise KeyError
+
+        return table
+
     @contextmanager
     def connect_to_db(self):
         try:
@@ -245,6 +261,13 @@ class AbstractDB(abc.ABC):
         finally:
             self.close_connection()
 
+    @contextmanager
+    def connect_to_table(self, table_name):
+        with self.connect_to_db():
+            table = self.generate_table_dict()[table_name]
+
+            yield table
+
     def execute(self, query):
         self.cursor.execute(query)
         self.cursor.commit()
@@ -280,10 +303,10 @@ class db(AbstractDB):
 
         self.connection = pyodbc.connect(
             r'DRIVER={' + self.DB_DRIVER + '};'
-            r'SERVER=' + self.DB_SERVER + ';'
-            r'DATABASE=' + self.DB_DATABASE + ';'
-            r'UID=' + self.DB_USERNAME + ';'
-            r'PWD=' + self.DB_PASSWORD + '',
+                                           r'SERVER=' + self.DB_SERVER + ';'
+                                                                         r'DATABASE=' + self.DB_DATABASE + ';'
+                                                                                                           r'UID=' + self.DB_USERNAME + ';'
+                                                                                                                                        r'PWD=' + self.DB_PASSWORD + '',
             timeout=1
 
         )
@@ -293,9 +316,9 @@ class db(AbstractDB):
     def connect_locally(self):
         self.connection = pyodbc.connect(
             r'DRIVER={' + self.DB_DRIVER + '};'
-            r'SERVER=' + self.DB_SERVER + ';'
-            r'DATABASE=' + self.DB_DATABASE + ';'
-            r'TRUSTED_CONNECTION=yes;',
+                                           r'SERVER=' + self.DB_SERVER + ';'
+                                                                         r'DATABASE=' + self.DB_DATABASE + ';'
+                                                                                                           r'TRUSTED_CONNECTION=yes;',
             timeout=1
             # r'PWD=' + self.DB_PASSWORD + '')
         )
@@ -340,7 +363,6 @@ class db(AbstractDB):
         return (foreign_keys)
 
 
-
 class Mysqldb(AbstractDB):
     def connect_locally(self):
         self.connection = MySQLdb.connect(host=self.DB_SERVER, user=self.DB_USERNAME, password=self.DB_PASSWORD,
@@ -358,6 +380,17 @@ class Mysqldb(AbstractDB):
         self.cursor = self.connection.cursor()
         print("DB connection established")
 
+    def create_new_db(self):
+        self.connection = MySQLdb.connect(host=self.DB_SERVER, user=self.DB_USERNAME,
+                                          password=self.DB_PASSWORD,
+                                          charset="utf8mb4", cursorclass=MySQLdb.cursors.DictCursor)
+
+        with self.connection.cursor() as cursor:
+            create_db_command = "CREATE DATABASE " + self.DB_DATABASE
+            cursor.execute(create_db_command)
+
+        self.connection.commit()
+
     def execute(self, query):
 
         self.cursor.execute(query)
@@ -379,6 +412,15 @@ class Mysqldb(AbstractDB):
 
 
 class PostgresDb(AbstractDBPostgres):
+
+    def connect_locally(self):
+        self.connection = psycopg2.connect(
+            host=self.DB_SERVER,
+            database=self.DB_DATABASE,
+            user=self.DB_USERNAME,
+            password=self.DB_PASSWORD)
+        self.cursor = self.connection.cursor()
+
     def connect_remotely(self):
         self.connection = psycopg2.connect(
             host=self.DB_SERVER,
@@ -390,6 +432,7 @@ class PostgresDb(AbstractDBPostgres):
 
     def execute(self, query):
         self.cursor.execute(query)
+        self.connection.commit()
 
         # return  [''.join(i) for i in self.cursor.fetchall()]
 
@@ -407,20 +450,16 @@ class PostgresDb(AbstractDBPostgres):
         return (table_dict)
 
 
-
-
-
 class BigQueryDb:
-    def __init__(self,credentials_path,project_id):
-        #super().__init__(None,None)
-        self.credentials_path=credentials_path
-        self.project_id=project_id
-
-        self.credentials = service_account.Credentials.from_service_account_file(self.credentials_path, scopes=["https://www.googleapis.com/auth/cloud-platform"],)
-
-        self.client = bigquery.Client(credentials=self.credentials, project=self.credentials.project_id,)
+    def __init__(self, credentials_path, project_id):
+        # super().__init__(None,None)
+        self.credentials_path = credentials_path
+        self.project_id = project_id
 
+        self.credentials = service_account.Credentials.from_service_account_file(self.credentials_path, scopes=[
+            "https://www.googleapis.com/auth/cloud-platform"], )
 
+        self.client = bigquery.Client(credentials=self.credentials, project=self.credentials.project_id, )
 
     def connect_remotely(self):
         print("Connect remotely")
@@ -437,9 +476,7 @@ class BigQueryDb:
 
         return df
 
-    def execute(self,query):
-
-
+    def execute(self, query):
         # query_job = self.client.query(
         #     """
         #     SELECT
@@ -453,23 +490,31 @@ class BigQueryDb:
         #     LIMIT 10"""
         # )
 
-
-
         query_job = self.client.query(query)
 
-
         results = query_job.result()  # Waits for job to complete.
 
         for row in results:
             print(row)
 
-            #print("{} : {} views".format(row.id, row.link,row.title))
-
+            # print("{} : {} views".format(row.id, row.link,row.title))
 
 
 class MongoDb(AbstractDBMongo):
+
+    def connect_locally(self):
+        self.connection = pymongo.MongoClient(
+            host=self.DB_SERVER,
+            authSource=self.DB_DATABASE,
+            username=self.DB_USERNAME,
+            password=self.DB_PASSWORD,
+            authMechanism='SCRAM-SHA-256'
+        )
+
+        self.database = self.connection.get_database(self.DB_DATABASE)
+        print('Connected locally')
+
     def connect_remotely(self):
-        # self.connection = MySQLdb.connect(self.DB_SERVER, self.DB_USERNAME, self.DB_PASSWORD, self.DB_DATABASE)
         self.connection = pymongo.MongoClient(
             "mongodb+srv://" + self.DB_USERNAME + ":" + self.DB_PASSWORD + "@" + self.DB_SERVER + "/" + self.DB_DATABASE + "?retryWrites=true&w=majority")
         print(self.connection.list_database_names())
@@ -484,9 +529,13 @@ class MongoDb(AbstractDBMongo):
     def get_all_tables(self):
         return self.database.list_collection_names()
 
-    def createTable(self, name):
+    def create_table(self, name):
         return self.database[name]
 
+    def createTable(self, name):
+        print("WARNING: `createTable` method will be deprecated in favor of `create_table`")
+        return self.create_table(name)
+
     def close_connection(self):
         self.connection.close()
         print("DB connection closed.")
@@ -536,7 +585,6 @@ class AbstractSelectable:
                 cleared_rows_list.append(list1)
         return (cleared_rows_list)
 
-
     def select_all(self):
         list1 = self.select("SELECT * FROM " + self.name)
         return (list1)
@@ -559,7 +607,6 @@ class Selectable(AbstractSelectable):  # Tables, views, and results of joins
 
 class MysqlSelectable(AbstractSelectable):
     def select(self, query):
-
         """TODO"""
         print(query)
         self.db1.execute(query)
@@ -578,7 +625,6 @@ class AbstractJoinable(AbstractSelectable):
         return (new_joinable)
 
 
-
 class Joinable(Selectable):
     pass
 
@@ -636,12 +682,12 @@ class AbstractTable(AbstractJoinable):
 
         return df_copy
 
-    def insert_from_df(self,df,batch=1,try_mode=False, debug_mode=False, adjust_df=False):
+    def insert_from_df(self, df, batch=1, try_mode=False, debug_mode=False, adjust_df=False):
+
         if adjust_df:
             df = self._adjust_df(df, debug_mode)
 
-        assert len(df.columns)+1==len(self.columns) #+1 because of id column
-
+        assert len(df.columns) + 1 == len(self.columns)  # +1 because of id column
 
         pd_nullable_dtypes = {pd.Int8Dtype(), pd.Int16Dtype(), pd.Int32Dtype(), pd.Int64Dtype(),
                               pd.UInt8Dtype(), pd.UInt16Dtype(), pd.UInt32Dtype(), pd.UInt64Dtype(),
@@ -657,12 +703,12 @@ class AbstractTable(AbstractJoinable):
         for column in list(df.columns):
             df.loc[pd.isna(df[column]), column] = "NULL"
 
-        rows=df.values.tolist()
-        for i,row in enumerate(rows):
-            for j,record in enumerate(row):
-                if type(record)==str:
-                    rows[i][j]="'"+record+"'"
-        self.insert(rows,batch=batch,try_mode=try_mode, debug_mode=False)
+        rows = df.values.tolist()
+        for i, row in enumerate(rows):
+            for j, record in enumerate(row):
+                if type(record) == str:
+                    rows[i][j] = "'" + record + "'"
+        self.insert(rows, batch=batch, try_mode=try_mode, debug_mode=False)
 
     def delete(self, where=None):
 
@@ -723,6 +769,74 @@ class PostgresTable(AbstractTable):
             print("Table " + self.name + " already exists:", e)
             print("Check the specification of table columns and their types")
 
+    def insert(self, rows, batch=1, replace_apostrophes=True, try_mode=False, debug_mode=False):
+        print("INSERTING!!!")
+        assert len(self.columns) == len(self.types)
+        print(self.types)
+        for k in range(len(rows)):
+            if k % batch == 0:
+                query = "INSERT INTO " + self.name + " ("
+                for i in range(1, len(self.columns)):
+                    if i < len(rows[k]) + 1:
+                        query += self.columns[i] + ","
+                if len(rows) < len(self.columns):
+                    print(len(self.columns) - len(rows), "columns were not specified")
+                query = query[:-1] + ") VALUES "
+
+            query += "("
+            for j in range(len(rows[k])):
+                if rows[k][j] == "NULL" or rows[k][j] == "'NULL'" or rows[k][j] == None or rows[k][
+                    j] == "None":  # NaN hodnoty
+                    if "int" in self.types[j + 1]:
+
+                        if replace_apostrophes:
+                            rows[k][j] = str(rows[k][j]).replace("'", "")
+                        query += "NULL,"
+                    else:
+                        query += "NULL,"
+                elif "nvarchar" in self.types[j + 1]:
+                    if replace_apostrophes:
+                        rows[k][j] = str(rows[k][j]).replace("'", "")
+                    query += "N'" + str(rows[k][j]) + "',"
+                elif "varchar" in self.types[j + 1]:
+                    if replace_apostrophes:
+                        rows[k][j] = str(rows[k][j]).replace("'", "")
+                    query += "'" + str(rows[k][j]) + "',"
+                elif self.types[j + 1] == "int":
+                    query += str(rows[k][j]) + ","
+                elif "datetime" in self.types[j + 1]:
+                    if replace_apostrophes:
+                        rows[k][j] = str(rows[k][j]).replace("'", "")
+                    query += "'" + str(rows[k][j]) + "',"
+                elif "date" in self.types[j + 1]:
+                    query += "'" + str(rows[k][j]) + "',"
+
+
+
+                else:
+                    query += str(rows[k][j]) + ","
+
+            query = query[:-1] + "),"
+            if k % batch == batch - 1 or k == len(rows) - 1:
+                query = query[:-1]
+
+                if debug_mode:
+                    print(query)
+
+                if not try_mode:
+                    self.db1.execute(query)
+                else:
+                    try:
+                        self.db1.execute(query)
+                    except Exception as e:
+
+                        print("Query", query, "Could not be inserted:", e)
+
+                        # Write to logs only in debug mode
+                        if debug_mode:
+                            with open("log.txt", "a") as file:
+                                file.write("Query " + str(query) + " could not be inserted:" + str(e) + "\n")
+
     @save_migration
     def add_column(self, column_name, column_type):
         assert len(column_name) > 1
@@ -763,23 +877,36 @@ class PostgresTable(AbstractTable):
 class MongoTable():
     def __init__(self, db, name, columns=[], types=[]):
         self.name = name
-        self.db = db
+        self.db1 = db
         print("==========================================")
         print(type(db))
         print(db)
-        self.collection = self.db.createTable(name)
+        self.collection = self.db1.create_table(name)
+
+    def create(self):
+        pass
 
     def drop(self):
         return self.collection.drop()
 
+    def update_collection(self):
+        self.collection = self.db1.create_table(self.name)
+
     def insert(self, document):
         return self.collection.insert_one(document)
 
-    def insertMore(self, documents):
+    def insert_more(self, documents):
         return self.collection.insert_many(documents)
 
+    def insertMore(self, documents):
+        print("WARNING: `insertMore` method will be deprecated in favor of `insert_more`")
+        return self.insert_more(documents)
+
     def select(self, query, columns={}):
 
+        if columns == '*':
+            columns = {}
+
         if (len(columns) == 0):
 
             return list(self.collection.find(query))
@@ -789,20 +916,32 @@ class MongoTable():
     def select_all(self, query={}):
         return list(self.collection.find(query))
 
-    def selectSort(self, query, fieldname, direction, columns={}):
+    def select_sort(self, query, fieldname, direction, columns={}):
         if (len(columns) == 0):
             return list(self.collection.find(query).sort(fieldname, direction))
         else:
             return list(self.collection.find(query, columns).sort(fieldname, direction))
 
+    def selectSort(self, query, fieldname, direction, columns={}):
+        print("WARNING: `selectSort` method will be deprecated in favor of `select_sort`")
+        return self.select_sort(query, fieldname, direction, columns)
+
     def delete(self, query={}):
+        self.collection = self.db1.create_table(self.name)
         return self.collection.delete_many(query)
 
-    def update(self, query, newvalues):
-        return self.collection.update(query, newvalues)
+    def update(self, newvalues, query):
+        return self.collection.update_many(query, newvalues)
+
+    def insert_from_df(self, dataframe):
+        dataframe = dataframe.replace({pd.NA: None})
+        dict_from_df = dataframe.to_dict('records')
+        # dict_from_df = dataframe.apply(lambda x : x.dropna().to_dict(),axis=1).tolist() #get rid of nans
+        return self.collection.insert_many(dict_from_df)
 
     def insertFromDataFrame(self, dataframe):
-        return self.collection.insert_many(dataframe.to_dict)
+        print("WARNING: `insertFromDataFrame` method will be deprecated in favor of `insert_from_df`")
+        return self.insert_from_df(dataframe)
 
     def select_to_df(self, query={}):
         print(type(pd.DataFrame(list(self.collection.find(query)))))
@@ -894,7 +1033,6 @@ class Table(Joinable, AbstractTable):
         types = temporary_table.get_all_types()
         return (cls(db1, name, columns, types))
 
-
     def get_all_columns(self):
         information_schema_table = Table(self.db1, 'INFORMATION_SCHEMA.COLUMNS')
         query = "SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME  = '" + self.name + "'"
@@ -923,7 +1061,7 @@ class Table(Joinable, AbstractTable):
             print("Table " + self.name + " already exists:", e)
             print("Check the specification of table columns and their types")
 
-    def insert(self,rows,batch=1,replace_apostrophes=True,try_mode=False, debug_mode=False):
+    def insert(self, rows, batch=1, replace_apostrophes=True, try_mode=False, debug_mode=False):
 
         assert len(self.columns) == len(self.types)
         for k in range(len(rows)):
@@ -959,9 +1097,9 @@ class Table(Joinable, AbstractTable):
                 else:
                     query += str(rows[k][j]) + ","
 
-            query=query[:-1]+"),"
-            if k%batch==batch-1 or k==len(rows)-1:
-                query=query[:-1]
+            query = query[:-1] + "),"
+            if k % batch == batch - 1 or k == len(rows) - 1:
+                query = query[:-1]
 
                 if debug_mode:
                     print(query)
@@ -980,16 +1118,12 @@ class Table(Joinable, AbstractTable):
                             with open("log.txt", "a") as file:
                                 file.write("Query " + str(query) + " could not be inserted:" + str(e) + "\n")
 
-
-
-
-
-    def get_foreign_keys_for_table(self,table_dict,foreign_keys):
-        #table_dict is in format from db function: generate_table_dict()
-        #foreign_keys are in format from db function: get_foreign_keys_columns()
-        parent_foreign_keys=[]
-        for i,fk in enumerate(foreign_keys):
-            if fk["parent_table"]==self.name:
+    def get_foreign_keys_for_table(self, table_dict, foreign_keys):
+        # table_dict is in format from db function: generate_table_dict()
+        # foreign_keys are in format from db function: get_foreign_keys_columns()
+        parent_foreign_keys = []
+        for i, fk in enumerate(foreign_keys):
+            if fk["parent_table"] == self.name:
 
                 try:
                     print(fk["parent_column_id"])
@@ -1009,7 +1143,6 @@ class MysqlTable(MysqlSelectable, AbstractTable):
         super().__init__(db1, name, columns)
         self.types = types
 
-
     def initialize_columns(self):
         information_schema_table = Table(self.db1, 'INFORMATION_SCHEMA.COLUMNS')
         query = "SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME  = '" + self.name + "';"
@@ -1031,18 +1164,17 @@ class MysqlTable(MysqlSelectable, AbstractTable):
 
     def get_all_types(self):
 
-        information_schema_table=Table(self.db1,'INFORMATION_SCHEMA.COLUMNS',['DATA_TYPE'],['nvarchar(50)'])
-        query="SELECT DATA_TYPE FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME  = '"+self.name+"'"
-        types=information_schema_table.select(query)
-        return(types)
-
+        information_schema_table = Table(self.db1, 'INFORMATION_SCHEMA.COLUMNS', ['DATA_TYPE'], ['nvarchar(50)'])
+        query = "SELECT DATA_TYPE FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME  = '" + self.name + "'"
+        types = information_schema_table.select(query)
+        return (types)
 
     @classmethod
-    def init_all_columns(cls,db1,name):
-        temporary_table=cls(db1,name)
-        columns=temporary_table.get_all_columns()
-        types=temporary_table.get_all_types()
-        return(cls(db1,name,columns,types))
+    def init_all_columns(cls, db1, name):
+        temporary_table = cls(db1, name)
+        columns = temporary_table.get_all_columns()
+        types = temporary_table.get_all_types()
+        return (cls(db1, name, columns, types))
 
     def get_last_id(self):
         """
@@ -1077,7 +1209,7 @@ class MysqlTable(MysqlSelectable, AbstractTable):
             print("Table " + self.name + " already exists:", e)
             print("Check the specification of table columns and their types")
 
-    def insert(self,rows,batch=1,replace_apostrophes=True,try_mode=False, debug_mode=False):
+    def insert(self, rows, batch=1, replace_apostrophes=True, try_mode=False, debug_mode=False):
         print("INSERTING!!!")
         assert len(self.columns) == len(self.types)
         print(self.types)
@@ -1093,7 +1225,8 @@ class MysqlTable(MysqlSelectable, AbstractTable):
 
             query += "("
             for j in range(len(rows[k])):
-                if rows[k][j] == "NULL" or rows[k][j] == "'NULL'" or rows[k][j] == None or rows[k][j] == "None":  # NaN hodnoty
+                if rows[k][j] == "NULL" or rows[k][j] == "'NULL'" or rows[k][j] == None or rows[k][
+                    j] == "None":  # NaN hodnoty
                     if "int" in self.types[j + 1]:
 
                         if replace_apostrophes:
@@ -1123,9 +1256,9 @@ class MysqlTable(MysqlSelectable, AbstractTable):
                 else:
                     query += str(rows[k][j]) + ","
 
-            query=query[:-1]+"),"
-            if k%batch==batch-1 or k==len(rows)-1:
-                query=query[:-1]
+            query = query[:-1] + "),"
+            if k % batch == batch - 1 or k == len(rows) - 1:
+                query = query[:-1]
 
                 if debug_mode:
                     print(query)
@@ -1142,12 +1275,12 @@ class MysqlTable(MysqlSelectable, AbstractTable):
                         # Write to logs only in debug mode
                         if debug_mode:
                             with open("log.txt", "a") as file:
-                                file.write("Query "+str(query)+" could not be inserted:"+str(e)+"\n")
+                                file.write("Query " + str(query) + " could not be inserted:" + str(e) + "\n")
 
-    def add_foreign_key(self,foreign_key):
-        parent_id=foreign_key['parent_id']
-        parent=foreign_key['parent']
-        query="ALTER TABLE "+self.name+" MODIFY "+parent_id+" INT UNSIGNED"
+    def add_foreign_key(self, foreign_key):
+        parent_id = foreign_key['parent_id']
+        parent = foreign_key['parent']
+        query = "ALTER TABLE " + self.name + " MODIFY " + parent_id + " INT UNSIGNED"
         print(query)
         self.db1.execute(query)
         query = "ALTER TABLE " + self.name + " ADD FOREIGN KEY (" + parent_id + ") REFERENCES " + parent + "(id)"
@@ -1196,13 +1329,12 @@ class XlsxDB:
     def __init__(self, name="new_db", config_file="config.ini"):
         self.name = name
 
-
         """
         db_details=read_connection_details(config_file)
         locally=True
         if db_details["LOCALLY"]=="False":
             locally=False
-            
+
         if locally:
             self.DB_SERVER=db_details["DB_SERVER"]
             self.DB_DATABASE=db_details["DB_DATABASE"]
@@ -1227,7 +1359,6 @@ class XlsxDB:
         # self.connection.close()
         # print("DB connection closed")
 
-
     def create_database(self):
         import os
         try:
@@ -1255,9 +1386,8 @@ class XlsxTable(AbstractTable):
             df = pd.DataFrame(columns=self.columns)
         return (df)
 
-
-    def insert_from_df(self,df,batch=1,try_mode=False, debug_mode=False):
-        assert len(df.columns)+1==len(self.columns) #+1 because of id column
+    def insert_from_df(self, df, batch=1, try_mode=False, debug_mode=False):
+        assert len(df.columns) + 1 == len(self.columns)  # +1 because of id column
 
         original_df = self.select_to_df()
 
@@ -1339,4 +1469,4 @@ def df_to_dict(df, column1, column2):
 
 def dict_to_df(dictionary, column1, column2):
     df = pd.DataFrame(list(dictionary.items()), columns=[column1, column2])
-    return (df)
+    return (df)
\ No newline at end of file
